@@ -15,7 +15,18 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Suite 500, Boston, MA 02110-1335, USA.            *
+ *                                                                         *
+ *   OpenSSL linking exception                                             *
+ *   --------------------------                                            *
+ *   If you modify this Program, or any covered work, by linking or        *
+ *   combining it with the OpenSSL project's "OpenSSL" library (or a       *
+ *   modified version of that library), containing parts covered by        *
+ *   the terms of OpenSSL/SSLeay license, the licensors of this            *
+ *   Program grant you additional permission to convey the resulting       *
+ *   work. Corresponding Source for a non-source form of such a            *
+ *   combination shall include the source code for the parts of the        *
+ *   OpenSSL library used as well as that of the covered work.             *
  ***************************************************************************/
 
 #include <openfetion.h>
@@ -26,7 +37,7 @@ static char* generate_contact_info_body(const char* userid);
 static char* generate_contact_info_by_no_body(const char* no , NumberType nt);
 static char* generate_set_mobileno_perssion(const char* userid , int show);
 static char* generate_set_displayname_body(const char* userid , const char* name);
-static char* generate_move_to_group_body(const char* userid , int buddylist);
+static char* generate_move_to_group_body(const char *userid, const char *groupids);
 static char* generate_delete_buddy_body(const char* userid);
 static char* generate_add_buddy_body(const char* no , NumberType notype
 								, int buddylist , const char* localname
@@ -36,10 +47,10 @@ static char* generate_handle_contact_request_body(const char* sipuri
 								, int buddylist , int result );
 static Contact* parse_handle_contact_request_response(const char* sipmsg);
 static Contact* parse_add_buddy_response(const char* sipmsg , int* statuscode);
-static void parse_set_displayname_response(User* user , const char* userid , const char* sipmsg);
 static int parse_set_mobileno_permission_response(User* user , const char* sipmsg);
 static Contact* parse_contact_info_by_no_response(const char* sipmsg);
 static int has_special_word(const char *in);
+static char *generate_group_body(const char *userid, const char *buddylist);
 
 Contact* fetion_contact_new()
 {
@@ -54,6 +65,7 @@ Contact* fetion_contact_new()
 	list->next = list;
 	return list;
 }
+
 void fetion_contact_list_append(Contact* cl , Contact* contact)
 {
 	cl->next->pre = contact;
@@ -118,6 +130,7 @@ void fetion_contact_list_remove(Contact *contact)
 	contact->next->pre = contact->pre;
 	contact->pre->next = contact->next;
 }
+
 void fetion_contact_list_free(Contact* contact)
 {
 	Contact *cl_cur , *del_cur;
@@ -130,6 +143,7 @@ void fetion_contact_list_free(Contact* contact)
 	}
 	free(contact);
 }
+
 int fetion_contact_subscribe_only(User* user)
 {
 	char *res, *body;
@@ -160,6 +174,7 @@ int fetion_contact_subscribe_only(User* user)
 	free(res);
 	return 0;
 }
+
 Contact* fetion_contact_get_contact_info(User* user , const char* userid)
 {
 	FetionSip* sip = user->sip;
@@ -171,9 +186,7 @@ Contact* fetion_contact_get_contact_info(User* user , const char* userid)
 	xmlNodePtr node;
 	contact = fetion_contact_list_find_by_userid(user->contactList , userid);
 	body = generate_contact_info_body(userid);
-	if(body == NULL){
-		return NULL;
-	}
+	if(body == NULL) return NULL;
 	fetion_sip_set_type(sip , SIP_SERVICE);
 	SipHeader* eheader = fetion_sip_event_header_new(SIP_EVENT_GETCONTACTINFO);
 	if(eheader == NULL){
@@ -293,28 +306,19 @@ int fetion_contact_set_mobileno_permission(User* user , const char* userid , int
 	int ret;
 	fetion_sip_set_type(sip , SIP_SERVICE);
 	eheader = fetion_sip_event_header_new(SIP_EVENT_SETCONTACTINFO);
-	if(eheader == NULL){
-		return -1;
-	}
+	if(eheader == NULL)	return -1;
 	fetion_sip_add_header(sip , eheader);
 	body = generate_set_mobileno_perssion(userid , show);
-	if(body == NULL){
-		return -1;
-	}
+	if(body == NULL) return -1;
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	if(res == NULL){
-		return -1;
-	}
+	if(res == NULL)	return -1;
 	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res) ; 
-	if(ret < 0)
-		return -1;
+	if(ret < 0)	return -1;
 
 	res = fetion_sip_get_response(sip);
-	if(res == NULL){
-		return -1;
-	}
+	if(res == NULL)	return -1;
 	ret = fetion_sip_get_code(res);
 	if(ret == 200){
 		parse_set_mobileno_permission_response(user , res);
@@ -367,48 +371,120 @@ int fetion_contact_set_displayname(User* user , const char* userid , const char*
 		return -1;
 	}
 }
-int fetion_contact_move_to_group(User* user , const char* userid , int buddylist)
+
+int fetion_contact_move_to_group(User *user, const char *userid, int old_bl, int new_bl)
 {
-	FetionSip* sip = user->sip;
-	SipHeader* eheader;
-	char *res , *body;
+	FetionSip *sip = user->sip;
+	SipHeader *eheader;
+	Contact    *cnt;
+	char        bls[1024] = { 0, };
+	char *res, *body;
 	int ret;
+
 	fetion_sip_set_type(sip , SIP_SERVICE);
 	eheader = fetion_sip_event_header_new(SIP_EVENT_SETCONTACTINFO);
-	if(eheader == NULL){
-		return -1;
-	}
 	fetion_sip_add_header(sip , eheader);
-	body = generate_move_to_group_body(userid , buddylist);
-	if(body == NULL){
-		return -1;
-	}
+
+	cnt = fetion_contact_list_find_by_userid(user->contactList, userid);
+	foreach_groupids(cnt->groupids) {
+		if(group_id == old_bl) continue;
+		sprintf(bls + strlen(bls), "%d;", group_id);
+	} end_groupids(cnt->groupids)
+	sprintf(bls + strlen(bls), "%d", new_bl);
+	sprintf(cnt->groupids, "%s", bls);
+
+	body = generate_move_to_group_body(userid, bls);
+
+	if(!body) return -1;
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	if(res == NULL){
-		return -1;
-	}
+	if(!res) return -1;
 	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res);
 
-	if(ret < 0)
-		return -1;
+	if(ret < 0)	return -1;
+
+	return 0;
+}
+
+int fetion_contact_copy_to_group(User *user, const char *userid, int buddylist)
+{
+	FetionSip *sip = user->sip;
+	SipHeader *eheader;
+	Contact   *cnt;
+	char *res, *body, newbl[48] = { 0,};
+	int ret;
+	fetion_sip_set_type(sip , SIP_SERVICE);
+	if (!(eheader = fetion_sip_event_header_new(SIP_EVENT_SETCONTACTINFO))) return -1;
+	fetion_sip_add_header(sip , eheader);
+	
+	cnt = fetion_contact_list_find_by_userid(user->contactList, userid);
+	
+	foreach_groupids(cnt->groupids) {
+		if(group_id == buddylist) return -1;
+		sprintf(newbl + strlen(newbl), "%d;", group_id);
+	} end_groupids(cnt->groupids)
+	sprintf(newbl + strlen(newbl), "%d", buddylist);
+	sprintf(cnt->groupids, "%s", newbl);
+	 
+	body = generate_group_body(userid, newbl);
+	res = fetion_sip_to_string(sip, body);
+	free(body);
+	if(!res) return -1;
+
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
+	free(res);
+
+	if(ret < 0)	return -1;
 
 	res = fetion_sip_get_response(sip);
-	if(res == NULL){
-		return -1;
-	}
+	if(!res) return -1;
+
 	ret = fetion_sip_get_code(res);
 	free(res);
 
-	if(ret == 200){
-		debug_info("Move buddy(%s) to group %d success" , userid , buddylist);
-		return 0;
-	}else{
-		debug_info("Move buddy(%s) to group %d failed" , userid , buddylist);
-		return -1;
-	}
+	return 0;
 }
+
+int fetion_contact_remove_from_group(User *user, const char *userid, int buddylist)
+{
+	FetionSip *sip = user->sip;
+	SipHeader *eheader;
+	Contact   *cnt;
+	char *res, *body, newbl[48] = { 0,};
+	int ret;
+	fetion_sip_set_type(sip , SIP_SERVICE);
+	if (!(eheader = fetion_sip_event_header_new(SIP_EVENT_SETCONTACTINFO))) return -1;
+	fetion_sip_add_header(sip , eheader);
+	
+	cnt = fetion_contact_list_find_by_userid(user->contactList, userid);
+	
+	foreach_groupids(cnt->groupids) {
+		if(group_id == buddylist) continue;
+		sprintf(newbl + strlen(newbl), "%d;", group_id);
+	} end_groupids(cnt->groupids)
+	if(newbl[strlen(newbl) - 1] == ';') newbl[strlen(newbl) - 1] = '\0';
+	sprintf(cnt->groupids, "%s", newbl);
+	 
+	body = generate_group_body(userid, newbl);
+	res = fetion_sip_to_string(sip, body);
+	free(body);
+	if(!res) return -1;
+
+	ret = tcp_connection_send(sip->tcp , res , strlen(res));
+	free(res);
+
+	if(ret < 0)	return -1;
+
+	res = fetion_sip_get_response(sip);
+	if(!res) return -1;
+
+	ret = fetion_sip_get_code(res);
+	free(res);
+
+	return 0;
+}
+
 int fetion_contact_delete_buddy(User* user , const char* userid)
 {
 	FetionSip* sip = user->sip;
@@ -417,19 +493,12 @@ int fetion_contact_delete_buddy(User* user , const char* userid)
 	int ret;
 	fetion_sip_set_type(sip , SIP_SERVICE);
 	eheader = fetion_sip_event_header_new(SIP_EVENT_DELETEBUDDY);
-	if(eheader == NULL){
-		return -1;
-	}
 	fetion_sip_add_header(sip , eheader);
 	body = generate_delete_buddy_body(userid);
-	if(body == NULL){
-		return -1;
-	}
+	if(body == NULL) return -1;
 	res = fetion_sip_to_string(sip , body);
 	free(body);
-	if(res == NULL){
-		return -1;
-	}
+	if(res == NULL)	return -1;
 #if 0
 	if(fetion_contact_del_localbuddy(user, userid) == -1)
 		return -1;
@@ -438,24 +507,14 @@ int fetion_contact_delete_buddy(User* user , const char* userid)
 	ret = tcp_connection_send(sip->tcp , res , strlen(res));
 	free(res);
 
-	if(ret < 0)
-		return -1;
+	if(ret < 0)	return -1;
 
 	res = fetion_sip_get_response(sip);
-	if(res == NULL){
-		return -1;
-	}
+	if(res == NULL)	return -1;
 	ret = fetion_sip_get_code(res);
 	free(res);
 
-	if(ret == 200){
-		fetion_contact_list_remove_by_userid(user->contactList , userid);
-		debug_info("Delete buddy(%s) success" , userid);
-		return 0;
-	}else{
-		debug_info("Delete buddy(%s) failed" , userid);
-		return -1;
-	}
+	return 0;
 }
 Contact* fetion_contact_add_buddy(User* user , const char* no
 								, NumberType notype , int buddylist
@@ -699,10 +758,10 @@ char* generate_set_displayname_body(const char* userid , const char* name)
 	xmlFreeDoc(doc);
 	return xml_convert(res);
 }
-char* generate_move_to_group_body(const char* userid , int buddylist)
+
+static char *generate_move_to_group_body(const char *userid, const char *groupids)
 {
 	char args[] = "<args></args>";
-	char bl[5];
 	xmlChar *res;
 	xmlDocPtr doc;
 	xmlNodePtr node;
@@ -711,12 +770,12 @@ char* generate_move_to_group_body(const char* userid , int buddylist)
 	node = xmlNewChild(node , NULL , BAD_CAST "contacts" , NULL);
 	node = xmlNewChild(node , NULL , BAD_CAST "contact" , NULL);
 	xmlNewProp(node , BAD_CAST "user-id" , BAD_CAST userid);
-	sprintf(bl , "%d" , buddylist);
-	xmlNewProp(node , BAD_CAST "buddy-lists" , BAD_CAST bl);
+	xmlNewProp(node , BAD_CAST "buddy-lists" , BAD_CAST groupids);
 	xmlDocDumpMemory(doc , &res , NULL);
 	xmlFreeDoc(doc);
 	return xml_convert(res);
 }
+
 char* generate_delete_buddy_body(const char* userid)
 {
 	char args[] = "<args></args>";
@@ -733,6 +792,7 @@ char* generate_delete_buddy_body(const char* userid)
 	xmlFreeDoc(doc);
 	return xml_convert(res);
 }
+
 char* generate_add_buddy_body(const char* no 
 		, NumberType notype , int buddylist 
 		, const char* localname , const char* desc , int phraseid)
@@ -768,33 +828,25 @@ char* generate_add_buddy_body(const char* no
 	xmlFreeDoc(doc);
 	return xml_convert(res);
 }
-static void parse_set_displayname_response(User* user , const char* userid , const char* sipmsg)
+
+static char *generate_group_body(const char *userid, const char *buddylist)
 {
-	char *pos;
-	Contact* contact;
-	xmlChar* res;
+	char args[] = "<args></args>";
+	xmlChar *res;
 	xmlDocPtr doc;
 	xmlNodePtr node;
-	contact = fetion_contact_list_find_by_userid(user->contactList , userid);
-	if(contact == NULL)
-	{
-		debug_error("Can not find user(%s) in contact list" , userid);
-		return;
-	}
-	pos = strstr(sipmsg , "\r\n\r\n") + 4;
-	doc = xmlParseMemory(pos , strlen(pos));
+	doc = xmlParseMemory(args , strlen(args));
 	node = xmlDocGetRootElement(doc);
-	node = node->xmlChildrenNode;
-	res = xmlGetProp(node , BAD_CAST "version");
-	memset(user->contactVersion, 0, sizeof(user->contactVersion));
-	strcpy(user->contactVersion , (char*)res);
-	xmlFree(res);
-	node = node->xmlChildrenNode;
-	res = xmlGetProp(node , BAD_CAST "local-name");
-	strcpy(contact->localname , (char*)res);
-	xmlFree(res);
+	node = xmlNewChild(node , NULL , BAD_CAST "contacts" , NULL);
+	node = xmlNewChild(node , NULL , BAD_CAST "contact" , NULL);
+
+	xmlNewProp(node , BAD_CAST "user-id" , BAD_CAST userid);
+	xmlNewProp(node , BAD_CAST "buddy-lists" , BAD_CAST buddylist);
+	xmlDocDumpMemory(doc , &res , NULL);
 	xmlFreeDoc(doc);
+	return xml_convert(res);
 }
+
 static int parse_set_mobileno_permission_response(User* user , const char* sipmsg)
 {
 	char *pos;
@@ -1099,7 +1151,7 @@ void fetion_contact_load(User *user, int *gcount, int *bcount)
 	}
 	sqlite3_free_table(sqlres);
 
-	sprintf(sql, "select * from contacts;");
+	sprintf(sql, "select * from contacts_2_2_0;");
 	if(sqlite3_get_table(db, sql, &sqlres, &nrows, &ncols, NULL)){
 		sqlite3_close(db);
 		return;
@@ -1132,6 +1184,7 @@ void fetion_contact_load(User *user, int *gcount, int *bcount)
 			strcpy(pos->carrier,	sqlres[start+18]);
 			pos->groupid = 			atoi(sqlres[start+19]);
 			pos->gender = 			atoi(sqlres[start+20]);
+			strcpy(pos->groupids,   sqlres[start+21]);
 		}
 		fetion_contact_list_append(user->contactList, pos);
 	}
@@ -1181,15 +1234,15 @@ void fetion_contact_save(User *user)
 		}
 	}
 	
-	sprintf(sql, "delete from contacts;");
+	sprintf(sql, "delete from contacts_2_2_0;");
 	if(sqlite3_exec(db, sql, NULL, NULL, NULL)){
-		sprintf(sql, "create table contacts (userId,"
+		sprintf(sql, "create table contacts_2_2_0 (userId,"
 						"sId,sipuri,localname,nickname,"
 						"impression,mobileno,devicetype,"
 						"portraitCrc,birthday,country,"
 						"province,city,identity,scoreLevel,"
 						"serviceStatus,carrierStatus,"
-						"relationStatus,carrier,groupid,gender);");
+						"relationStatus,carrier,groupid,gender,groupids);");
 		if(sqlite3_exec(db, sql, NULL, NULL, NULL)){
 			debug_error("create table contacts:%s", sqlite3_errmsg(db));
 			sqlite3_close(db);
@@ -1197,10 +1250,10 @@ void fetion_contact_save(User *user)
 		}
 	}
 	foreach_contactlist(user->contactList, pos){
-		snprintf(sql, sizeof(sql)-1, "insert into contacts "
+		snprintf(sql, sizeof(sql)-1, "insert into contacts_2_2_0 "
 				"values ('%s','%s','%s','%s','%s','%s',"
 				"'%s','%s','%s','%s','%s','%s','%s','%d',%d,"
-				"%d,%d,%d,'%s',%d,%d);",
+				"%d,%d,%d,'%s',%d,%d,'%s');",
 				pos->userId, pos->sId, pos->sipuri,
 				has_special_word(pos->localname) ? "": pos->localname,
 			   	has_special_word(pos->nickname)? "": pos->nickname,
@@ -1211,7 +1264,7 @@ void fetion_contact_save(User *user)
 				pos->city, pos->identity, pos->scoreLevel,
 				pos->serviceStatus, pos->carrierStatus,
 				pos->relationStatus, pos->carrier,
-				pos->groupid, pos->gender);
+				pos->groupid, pos->gender, pos->groupids);
 		if(sqlite3_exec(db, sql, NULL, NULL, NULL))
 			debug_error("insert contact %s:%s\n%s",
 					pos->userId, sqlite3_errmsg(db), sql);
@@ -1241,7 +1294,7 @@ void fetion_contact_update(User *user, Contact *contact)
 		return;
 	}
 
-	snprintf(sql, sizeof(sql)-1, "update contacts set "
+	snprintf(sql, sizeof(sql)-1, "update contacts_2_2_0 set "
 			"userId='%s',sId='%s',sipuri='%s',"
 			"localname='%s',nickname='%s',"
 			"impression='%s',mobileno='%s',"
@@ -1251,7 +1304,7 @@ void fetion_contact_update(User *user, Contact *contact)
 			"identity=%d,scoreLevel=%d,"
 			"serviceStatus=%d,carrierStatus=%d,"
 			"relationStatus=%d,carrier='%s',"
-			"groupid=%d,gender=%d where userId='%s'",
+			"groupid=%d,gender=%d,groupids='%s' where userId='%s'",
 			contact->userId, contact->sId, contact->sipuri,
 			contact->localname, contact->nickname,
 			contact->impression, contact->mobileno,
@@ -1260,27 +1313,27 @@ void fetion_contact_update(User *user, Contact *contact)
 			contact->city, contact->identity, contact->scoreLevel,
 			contact->serviceStatus, contact->carrierStatus,
 			contact->relationStatus, contact->carrier,
-			contact->groupid, contact->gender, contact->userId);
+			contact->groupid, contact->gender, contact->groupids, contact->userId);
 
 	if(sqlite3_exec(db, sql, NULL, NULL, NULL)){
 		debug_error("update contact %s:%s", contact->userId, sqlite3_errmsg(db));
-		sprintf(sql, "create table contacts (userId,"
+		sprintf(sql, "create table contacts_2_2_0 (userId,"
 					"sId,sipuri,localname,nickname,"
 					"impression,mobileno,devicetype,"
 					"portraitCrc,birthday,country,"
 					"province,city,identity,scoreLevel,"
 					"serviceStatus,carrierStatus,"
-					"relationStatus,carrier,groupid,gender);");
+					"relationStatus,carrier,groupid,gender,groupids);");
 		if(sqlite3_exec(db, sql, NULL, NULL, NULL)){
 			debug_error("create table contacts:%s", sqlite3_errmsg(db));
 			sqlite3_close(db);
 			return;
 		}
 
-		snprintf(sql, sizeof(sql)-1, "insert into contacts "
+		snprintf(sql, sizeof(sql)-1, "insert into contacts_2_2_0 "
 					"values ('%s','%s','%s','%s','%s','%s',"
 					"'%s','%s','%s','%s','%s','%s','%s','%d',%d,"
-					"%d,%d,%d,'%s',%d,%d);",
+					"%d,%d,%d,'%s',%d,%d,'%s');",
 					contact->userId, contact->sId, contact->sipuri,
 					contact->localname, contact->nickname,
 					contact->impression, contact->mobileno,
@@ -1289,7 +1342,7 @@ void fetion_contact_update(User *user, Contact *contact)
 					contact->city, contact->identity, contact->scoreLevel,
 					contact->serviceStatus, contact->carrierStatus,
 					contact->relationStatus, contact->carrier,
-					contact->groupid, contact->gender);
+					contact->groupid, contact->gender, contact->groupids);
 
 		if(sqlite3_exec(db, sql, NULL, NULL, NULL)){
 			debug_error("insert contacts:%s", sqlite3_errmsg(db));
@@ -1313,7 +1366,7 @@ int fetion_contact_del_localbuddy(User *user, const char *userid)
 		return -1;
 	}
 
-	sprintf(sql, "delete from contacts where "
+	sprintf(sql, "delete from contacts_2_2_0 where "
 			"userid='%s';", userid);
 	if(sqlite3_exec(db, sql, NULL, NULL, NULL)){
 		debug_error("failed to delete localbuddy:%s",sqlite3_errmsg(db));
